@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 """
-Copyright: Sang Han 2014
+Copyright: 2014 Sang Han
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -14,26 +14,34 @@ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 """
+import sys
 
-from sys import exit
-from argparse import ArgumentParser
+from argparse import ArgumentParser, RawDescriptionHelpFormatter, FileType, REMAINDER
+from functools import partial
 
 from .__version__ import __version__, __build__
 from .translator import translator
-from .coroutines import chunk, spool, source, set_task
+from .coroutines import spool, source, set_task
 from .languages import print_table, translation_table
 
 __all__ = []
 
+class TermWidthHelpFormatter(RawDescriptionHelpFormatter):
+
+    def __init__(self,  *args, **kwargs):
+        super(TermWidthHelpFormatter, self).__init__(
+            width=120, max_help_position=35, *args, **kwargs
+        )
+
 def command_line():
 
     description = 'A simple translation command line utility'
-    epilog      = ' '.join(sorted(translation_table('en').keys()))
     version     = ' '.join([__version__, __build__])
+    table       = sorted(translation_table('en').keys())
 
     codes = ArgumentParser(add_help=False)
     codes.add_argument(
-        '-l',
+        '-l', '--list',
         nargs='?',
         default=False,
         const='en',
@@ -57,7 +65,12 @@ def command_line():
         parents=[codes],
         prog='translate',
         description=description,
-        epilog=epilog
+        formatter_class=TermWidthHelpFormatter,
+    )
+    parser.add_argument(
+        '--translit',
+        action='store_true',
+        help='Print out the transliteration of the text',
     )
     parser.add_argument(
         '-v', '--version',
@@ -66,11 +79,25 @@ def command_line():
     )
     parser.add_argument(
         'source',
+        nargs='?',
+        default=None,
         help='Source language code',
+        metavar='source',
     )
     parser.add_argument(
         'dest',
-        help='Destination language code'
+        type=str,
+        choices=table,
+        help='Destination language code',
+        metavar='target',
+    )
+    parser.add_argument(
+        'text',
+        nargs='?',
+        default=sys.stdin,
+        type=FileType('r'),
+        help='Text file',
+        metavar='file',
     )
 
     return parser.parse_args()
@@ -80,11 +107,11 @@ def main():
     '''
     Main Entry point for translator and argument parser
     '''
-    args = command_line()
+    args      = command_line()
+    translate = partial(translator, args.source, args.dest,
+                        version=' '.join([__version__, __build__]))
 
-    source(spool(chunk(set_task(translator, args.source, args.dest))))
-
-    return
+    return source(spool(set_task(translate, translit=args.translit)), args.text)
 
 if __name__ == '__main__':
-    exit(main())
+    sys.exit(main())
