@@ -12,17 +12,15 @@ consumer/producers
 
 from __future__ import print_function
 
-from sys import stdin, stdout
+import operator
+import sys
+
 from functools import wraps, partial, reduce
 from multiprocessing import cpu_count
-from multiprocessing.dummy import Pool as ThreadPool
 
-from six.moves.urllib.parse import quote
+from concurrent.futures import ThreadPoolExecutor
 
-__all__ = 'coroutine', 'chunk', 'spool', 'source', 'set_task', 'write_stream'
-
-# TODO: Get rid of this global variable
-MAX_WORK = cpu_count() ** 2
+__all__ = 'coroutine', 'spool', 'source', 'set_task', 'write_stream'
 
 def coroutine(func):
     """
@@ -61,24 +59,51 @@ def coroutine(func):
 
     return initialization
 
-def write_stream(script):
+
+def accumulator(init, update):
+    """
+    Generic accumulator function.
+
+    :param init:  Initial Value
+    :param update: Value to accumulate
+
+    :return: Combined Values
+    """
+    return (
+        init + len(update)
+            if isinstance(init, int) else
+        init + update
+    )
+
+
+def write_stream(script, output='trans'):
     """
     :param script: Translated Text
     :type script: Iterable
 
     :return None:
     """
-    for trans in script:
+    printer   = partial(print, file=sys.stdout, end='')
+    sentences = script.get('sentences', None)
 
-        for line in trans['sentences']:
-            stdout.write(line['trans'])
+    assert(output in sentences[0])
 
-        stdout.write('\n')
+    for line in sentences:
+        # I hate you unicode I hate you I hate you I hate you
+        if isinstance(line[output], str):
+            printer(line[output])
+        else:
+            printer(line[output].encode('UTF-8'))
 
-    return None
+    printer('\n')
 
+    return sys.stdout.flush()
+
+
+
+# TODO: Get rid of all this context crap
 @coroutine
-def set_task(translation_function, source, dest):
+def set_task(translator, translit=False):
     """
     Task Setter Coroutine
 
@@ -88,54 +113,44 @@ def set_task(translation_function, source, dest):
     :param translation_function: Translator
     :type translation_function: Function
 
-    :param source: Source Language Code
-    :type source: String
-
-    :param dest: Destination Language Code
-    :type dest: String
+    :param translit: Transliteration Switch
+    :type: Bool
     """
-    tasks     = []
-    workers   = ThreadPool(MAX_WORK)
-    translate = partial(translation_function, source, dest)
+    # Initialize Task Queue
+    task    = str()
+    queue   = list()
 
-    try:
-        while True:
-            tasks = yield
-            write_stream(workers.map(translate, tasks))
+    # Callable Objects
+    first   = operator.itemgetter(0)
+    done    = operator.methodcaller('done')
+    result  = operator.methodcaller('result')
 
-    except GeneratorExit:
-        workers.close()
-        workers.join()
+    # Function Partial
+    output  = ('translit' if translit else 'trans')
+    stream  = partial(write_stream, output=output)
 
-@coroutine
-def chunk(task):
-    """
-    Chunk text into a queue and send by copy to a task setter coroutine.
-    Once a queue has been built it is sent by copy and then destroyed.
-
-    :param task: Task setter
-    :type task: Coroutine
-    """
-    queue = []
+    # Worker Thread Pool
+    maxwork = cpu_count() * 32
+    workers = ThreadPoolExecutor(maxwork)
 
     try:
         while True:
 
-            while len(queue) < MAX_WORK:
-                line = yield
-                queue.append(line)
+            task = yield
+            queue.append(workers.submit(translator, task))
 
-            task.send(queue)
-            queue = []
+            while queue and done(first(queue)):
+                stream(result(queue.pop()))
 
     except GeneratorExit:
-        task.send(queue)
-        task.close()
+        workers.shutdown(wait=True)
+        list(map(stream,(map(result,queue))))
 
 @coroutine
-def spool(iterable, maxlen=1500):
+def spool(iterable, maxlen=1200):
     """
-    Consumes text streams and spools them together for more io efficient processes.
+    Consumes text streams and spools them together for more io
+    efficient processes.
 
     :param iterable: Sends text stream for further processing
     :type iterable: Coroutine
@@ -143,13 +158,8 @@ def spool(iterable, maxlen=1500):
     :param maxlen: Maximum query string size
     :type maxlen: Integer
     """
-    words, text = 0, ''
-
-    accumulator = lambda a, x: (
-        a + len(quote(x).encode('utf-8'))
-            if isinstance(a, int) else
-        a + x
-    )
+    words = int()
+    text  = str()
 
     try:
         while True:
@@ -160,27 +170,30 @@ def spool(iterable, maxlen=1500):
                 words  = reduce(accumulator, stream, words)
 
             iterable.send(text)
-
-            words, text = 0, ''
+            words = int()
+            text  = str()
 
     except GeneratorExit:
         iterable.send(text)
         iterable.close()
 
-# TODO: Implement FileIO
-def source(target):
+
+def source(target, inputstream=sys.stdin):
     """
     Coroutine starting point. Produces text stream and forwards to consumers
 
     :param target: Target coroutine consumer
     :type target: Coroutine
     """
+    for line in inputstream:
 
-    if stdin.isatty():
-        # target.send(args.file)
-        return
+        while len(line) > 500:
+            init, sep, line = line.partition(' ')
+            assert(len(init) <= 500)
+            target.send(''.join([init, sep]))
 
-    for line in stdin:
         target.send(line)
 
-    target.close()
+    inputstream.close()
+
+    return target.close()
