@@ -9,18 +9,16 @@ module deals with the client side logic of pushing the translation request
 to the the server.
 """
 
-import json
 import functools
+import requests
+import codecs
 
-from six.moves.urllib.request import urlopen, Request
-from six.moves.urllib.parse import urlencode
-
-from .__version__ import __version__ as version
-from .__version__ import __build__ as build
+from requests import Request, Session
+from requests.adapters import HTTPAdapter
 
 __all__ = 'push_url', 'translator'
 
-def push_url(site):
+def push_url(interface):
     '''
     Decorates a function returning the url of translation API.
     Creates and maintains HTTP connection state
@@ -28,44 +26,36 @@ def push_url(site):
     Returns a dict response object from the server containing the translated
     text and metadata of the request body
 
-    :param site: translator
-    :type site: Function
-
-    :return: HTTP Response
-    :rtype: Function
+    :param interface: Callable Request Interface
+    :type interface: Function
     '''
 
-    @functools.wraps(site)
+    @functools.wraps(interface)
     def connection(*args, **kwargs):
         """
-        Inner function that makes the http connection.
+        Extends and wraps a HTTP interface.
+
+        :return: Response Content
+        :rtype: Dictionary
         """
-        stream  = ''
-        req     = None
-        agent   = 'py-translate v{} {}'.format(version, build)
-        charset = 'utf-8'
+        session = Session()
+        session.mount('http://',  HTTPAdapter(max_retries=2))
+        session.mount('https://', HTTPAdapter(max_retries=2))
 
-        headers = {
-              'User-Agent': agent,
-            'Content-Type': 'application/json; charset={}'.format(charset)
-        }
-        url     = site(*args, **kwargs)
-        request = Request(url, headers=headers)
+        request  = Request(**interface(*args, **kwargs))
+        prepare  = session.prepare_request(request)
+        response = session.send(prepare, verify=True)
 
-        try:
-            req    = urlopen(request)
-            stream = req.read().decode(charset)
+        if response.status_code != requests.codes.ok:
+            response.raise_for_status()
 
-        finally:
-            req.close()
-
-        return json.loads(stream)
+        return response.json()
 
     return connection
 
 
 @push_url
-def translator(source, target, phrase):
+def translator(source, target, phrase, version='0.0 test', charset='utf-8'):
     """
     Returns the url encoded string that will be pushed to the translation
     server for parsing.
@@ -91,20 +81,20 @@ def translator(source, target, phrase):
     :param phrase: Text body string that will be url encoded and translated
     :type phrase: String
 
-    :return: url
-    :rtype: String
+    :return: Request Interface
+    :rtype: Dictionary
     """
-    base   = 'http://translate.google.com/translate_a/t'
-    params = urlencode(
-        {
-        'client': 'webapp',
-            'ie': 'utf-8',
-            'oe': 'utf-8',
-            'sl': source,
-            'tl': target,
-             'q': phrase,
-        }
-    )
-    url    = '?'.join([base, params])
 
-    return url
+    url     = 'https://translate.google.com/translate_a/t?'
+    agent   = 'User-Agent',   'py-translate v{}'.format(version)
+    content = 'Content-Type', 'application/text; charset={}'.format(charset)
+
+    params  = {'client': 'webapp', 'ie': charset, 'oe': charset,
+                   'sl':   source, 'tl':  target,  'q': phrase, }
+
+    request = {'method': 'GET',
+                  'url': url,
+               'params': params,
+              'headers': dict([agent, content])}
+
+    return request
